/*
 * AttrDef.java
 *
 * Created on April 14, 2003, 11:47 AM
 */

package dbmapper;

import java.io.Serializable;

/**
 * This class maintains the definition of a single attribute
 * within a class.
 * @author Larry
 */
public class AttrDef implements Serializable, MappedReference, Cloneable  {
 
    /** The name of this attribute
     */    
    private String name = null;
    /** The type of this attribute
     */    
    private String type = null;
    
    /** A reference to the object we are mapped to
     */    
    private MappedReference obj = null;
    /** The type of mapping our reference is
     */    
    private int mapType = ClassTableMap.MAP_TYPE_NA;
    
    /** The properties of our mapping
     */    
    private MappingProperties mapProp = null;
    
    /** Creates a new instance of AttrDef */
    public AttrDef() {
    }
    /**
     * Create a new AttrDef with the specified name and type
     * @param nam The name of the attribute
     * @param typ The type of the attribute
     */    
    public AttrDef(String nam, String typ) {
        name=nam;
        type=typ;
    }
    
    /** get a Pretty Print string representation of this attribute
     * @return The pretty print string
     */    
    public String getPPstr(){ return name + " (" + type + ")"; }
    /**
     * get the name of this attribute
     * @return attribute name
     */    
    public String getName(){return name;}
    /**
     * get the type of this attribute
     * @return The attribute type
     */    
    public String getType(){return type;}
        
    /**
     * The entity this attribute is mapped to
     * @param o The object we map to
     */    
    public void setMappedTo(MappedReference o){
        obj=o;
    }
    
    /**
     * Get the object we are mapped to
     * @return Object we are mapped to
     */    
    public MappedReference getMappedTo(){
        return obj;
    }
    
    /**
     * Are we currently mapped to another object?
     * @return t/f if we are mapped or not
     */    
    public boolean isMapped(){
        if ( obj != null ) return true;
        return false;
    }
    
    /**
     * Set the type of mapping we are defined to be
     * @see dbmapper.ClassTableMap#
     * @param type The mapping type we are
     */    
    public void setMapType(int type){
        mapType = type;
    }
    /**
     * Get our mapping type
     * @see dbmapper.ClassTableMap#
     * @return The type of our mapping
     */    
    public int getMapType(){
        return mapType;
    }
    
    /**
     * Get the mapping properties for our mapping link
     * @return The mapping props
     */    
    public MappingProperties getMapProp(){ return mapProp; }
    /**
     * Set the mapping properties for our mapping reference
     * @param prop The mapping properties
     */    
    public void setMapProp(MappingProperties prop){ mapProp=prop; }
}
