/*
 * Config.java
 *
 * Created on April 10, 2003, 5:17 PM
 */

package dbmapper;

import java.util.Properties;
import java.io.File;
import java.io.FileOutputStream;


/** This class wraps all the configuration options into a single class
 * for simple access.
 * @author Larry
 */
public class Config {
    
    /** The jdbc driver class
     */    
    private String jdbcDriver;
    public void setJdbcDriver(String j){jdbcDriver=j;}
    public String getJdbcDriver(){return jdbcDriver;}
    
    /** the DB url
     */    
    private String url;
    public void setUrl(String u){url=u;}
    public String getUrl(){return url;}
    
    /** The db user id
     */    
    private String userid;
    public void setUserid(String u){userid=u;}
    public String getUserid(){return userid;}
    
    /** The db access password
     */    
    private String password;
    public void setPassword(String p){password=p;}
    public String getPassword(){return password;} 
    
    /** The db schema to use (needed?)
     */    
    private String schema;
    public void setSchema(String s){schema=s;}
    public String getSchema(){return schema;}
    
    /** The jar file containing the classes to map
     */    
    private String jar;
    public void setJar(String j){jar=j;}
    public String getJar(){return jar;}
    
    /** The filter to apply to classes in the jar
     */    
    private String jarfilter;
    public void setJarFilter(String f){jarfilter=f;}
    public String getJarFilter(){return jarfilter;}
    
    /** Should we always do a fresh load of clases/tables upon startup.
     */    
    private boolean alwaysload;
    public void setAlwaysLoad(boolean flag){alwaysload=flag;}
    public boolean getAlwaysLoad(){return alwaysload;}
    
    /** Creates a new instance of Config */
    public Config() {
    }
    
    /** Set the configuration options from system properties
     */    
    public void setFromProperties(){
        setJdbcDriver(System.getProperty("dbmapper.jdbcdriver"));
        setUrl(System.getProperty("dbmapper.url"));
        setUserid(System.getProperty("dbmapper.userid"));
        setPassword(System.getProperty("dbmapper.password"));
        setSchema(System.getProperty("dbmapper.schema"));
        setJar(System.getProperty("dbmapper.jar"));
        setJarFilter(System.getProperty("dbmapper.jarfilter"));
        setAlwaysLoad(Boolean.valueOf(System.getProperty("dbmapper.alwaysload")).booleanValue());
    }
    
    /** Save the configuration options to a property file
     * @param fileName The file to write the properties out to
     * @throws Exception If a problem occurs during open, writing, or closing
     */    
    public void saveAsProperties(String fileName) throws Exception{
        Properties props=new Properties();
        props.setProperty("dbmapper.jdbcdriver", getJdbcDriver());
        props.setProperty("dbmapper.url", getUrl());
        props.setProperty("dbmapper.userid", getUserid());
        props.setProperty("dbmapper.password", getPassword());
        props.setProperty("dbmapper.schema", getSchema());
        props.setProperty("dbmapper.jar", getJar());
        props.setProperty("dbmapper.jarfilter", getJarFilter());
        props.setProperty("dbmapper.alwaysload", (new Boolean(getAlwaysLoad()).toString()));
        props.store(new FileOutputStream(fileName), "DbMapper Properties file");
        return;
    }
}
