/*
 * MapAssocs.java
 *
 * Created on April 10, 2003, 1:19 PM
 */

package dbmapper;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import javax.swing.JFileChooser;
import javax.swing.DefaultListModel;

/** This class creates the main class to mapping form. From this
 * class, the user can choose a class, choose a table, and
 * begin the mapping process.<p>
 * Other functions in this class:<p>
 * <ul>
 * <li>Save/Restore a state file
 * <li>Edit Configuration settings
 * <li>Load the classes and tables
 * <li>View the generated xml file
 * <li>Save the generated xml to a single file
 * <li>Save the generated xml to an exploded hierarchy (like compiler)
 * </UL>
 *
 * @author Larry
 */
public class MapAssocs extends javax.swing.JFrame {
    
    /** The sourceprocessor to use
     */    
    SourceProcessor processor=null;
    
    /** The ListModel for the class list
     */    
    DefaultListModel classListModel = null;
    /** The ListModel for the table List
     */    
    DefaultListModel tableListModel = null;
    
    /** Set the processor to use for this class
     * @param processor The processor
     */    
    public void setProcessor(SourceProcessor processor){
        this.processor=processor;
        refresh();
    }
    
    /** Clear the lists and reload with selection settings
     */    
    public void refresh(){
        if ( processor == null || !processor.isLoaded() ) return;
        classListModel.clear();
        String[] classListArr=processor.getClassList();
        for ( int i=0; i<classListArr.length; i++){
            classListModel.addElement(classListArr[i]);
        }
        tableListModel.clear();
        String[] tableListArr=processor.getTableList();
        for ( int i=0; i<tableListArr.length; i++){
            tableListModel.addElement(tableListArr[i]);
        }
        classList.repaint();
        tableList.repaint();
    }
    
    /** Creates new form MapAssocs */
    public MapAssocs() {
        initComponents();
        classListModel = new DefaultListModel();
        classList.setModel(classListModel);
        tableListModel = new DefaultListModel();
        tableList.setModel(tableListModel);
        // set the list cell renderer
        //MappedCellRenderer renderer = new MappedListCellRenderer();
        //classList.setCellRenderer(renderer);
        //tableList.setCellRenderer(renderer);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        tableScrollerPane = new javax.swing.JScrollPane();
        tableList = new javax.swing.JList();
        classLab = new javax.swing.JLabel();
        dbtabLab = new javax.swing.JLabel();
        assocChooTitLab = new javax.swing.JLabel();
        showLab1 = new javax.swing.JLabel();
        showLab2 = new javax.swing.JLabel();
        classModCombo = new javax.swing.JComboBox();
        dbtabModCombo = new javax.swing.JComboBox();
        mapSelectedButt = new javax.swing.JButton();
        showMappingButt = new javax.swing.JButton();
        saveMappingButt = new javax.swing.JButton();
        configButt = new javax.swing.JButton();
        quitButt = new javax.swing.JButton();
        classScrollPane = new javax.swing.JScrollPane();
        classList = new javax.swing.JList();
        saveStateButt = new javax.swing.JButton();
        loadStateButt = new javax.swing.JButton();
        editExistButt = new javax.swing.JButton();
        saveHierMappingButt = new javax.swing.JButton();
        loadDbCl = new javax.swing.JButton();

        getContentPane().setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        tableList.setBorder(new javax.swing.border.EtchedBorder());
        tableList.setFont(new java.awt.Font("Arial", 0, 12));
        tableList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        tableList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        tableList.setVisibleRowCount(10);
        tableScrollerPane.setViewportView(tableList);

        getContentPane().add(tableScrollerPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(450, 60, 240, 220));

        classLab.setText("Classes:");
        getContentPane().add(classLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 40, -1, -1));

        dbtabLab.setText("Database Tables");
        getContentPane().add(dbtabLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(480, 40, 100, -1));

        assocChooTitLab.setFont(new java.awt.Font("Century Schoolbook", 1, 14));
        assocChooTitLab.setText("Assosciation Chooser");
        getContentPane().add(assocChooTitLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(260, 10, 160, -1));

        showLab1.setText("Show:");
        getContentPane().add(showLab1, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 290, -1, -1));

        showLab2.setText("Show:");
        getContentPane().add(showLab2, new org.netbeans.lib.awtextra.AbsoluteConstraints(460, 290, 40, -1));

        classModCombo.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "All", "Unmapped", "Mapped" }));
        classModCombo.setSelectedIndex(1);
        classModCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                classModComboActionPerformed(evt);
            }
        });

        getContentPane().add(classModCombo, new org.netbeans.lib.awtextra.AbsoluteConstraints(80, 290, 120, -1));

        dbtabModCombo.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "All", "Mapped", "Unmapped" }));
        dbtabModCombo.setSelectedIndex(2);
        dbtabModCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dbtabModComboActionPerformed(evt);
            }
        });

        getContentPane().add(dbtabModCombo, new org.netbeans.lib.awtextra.AbsoluteConstraints(510, 290, 120, -1));

        mapSelectedButt.setText("Map Selected");
        mapSelectedButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mapSelectedButtActionPerformed(evt);
            }
        });

        getContentPane().add(mapSelectedButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 80, -1, -1));

        showMappingButt.setText("ShowMapping");
        showMappingButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showMappingButtActionPerformed(evt);
            }
        });

        getContentPane().add(showMappingButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 360, -1, -1));

        saveMappingButt.setText("SaveMonolithicMapFile...");
        saveMappingButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveMappingButtActionPerformed(evt);
            }
        });

        getContentPane().add(saveMappingButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(510, 400, -1, -1));

        configButt.setText("Configure...");
        configButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                configButtActionPerformed(evt);
            }
        });

        getContentPane().add(configButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 400, -1, -1));

        quitButt.setText("Quit");
        quitButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quitButtActionPerformed(evt);
            }
        });

        getContentPane().add(quitButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(350, 400, -1, -1));

        classList.setFont(new java.awt.Font("Arial", 0, 12));
        classList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "b", "b", "b", "b", "b", "b", "b", "b", "b", "b", "b", "b", "b", "b", "b", "b" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        classList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        classScrollPane.setViewportView(classList);

        getContentPane().add(classScrollPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 60, 290, 220));

        saveStateButt.setText("Save State...");
        saveStateButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveStateButtActionPerformed(evt);
            }
        });

        getContentPane().add(saveStateButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 360, -1, -1));

        loadStateButt.setText("Load State...");
        loadStateButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadStateButtActionPerformed(evt);
            }
        });

        getContentPane().add(loadStateButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(130, 360, -1, -1));

        editExistButt.setText("edit Existing");
        editExistButt.setToolTipText("Select a Mapped class to modify it's mappings");
        editExistButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editExistButtActionPerformed(evt);
            }
        });

        getContentPane().add(editExistButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(330, 250, -1, -1));

        saveHierMappingButt.setText("SaveMapHier...");
        saveHierMappingButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveHierMappingButtActionPerformed(evt);
            }
        });

        getContentPane().add(saveHierMappingButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(540, 360, -1, -1));

        loadDbCl.setText("Load db/classes");
        loadDbCl.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadDbClActionPerformed(evt);
            }
        });

        getContentPane().add(loadDbCl, new org.netbeans.lib.awtextra.AbsoluteConstraints(130, 400, -1, -1));

        pack();
    }//GEN-END:initComponents

    /** Callback when the "Load db/table" button is pressed. Causes a fresh
     * reload of database tables and java classes.
     * @param evt callback event
     */    
    private void loadDbClActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadDbClActionPerformed
        // load db&classes
        Config conf=new Config();
        conf.setFromProperties();
        try {
            processor.load(conf);
            refresh();
        }catch (Exception e){
            ErrorPopup ep=new ErrorPopup(this, true);  
            ep.popup("Unable to load tables/classes. Exception: " + e.toString());
            return;
        }
    }//GEN-LAST:event_loadDbClActionPerformed

    /** Callback when "Save mapping hier" button is called to save
     * the xml file in an exploded one file per class hierarchy.<p>
     * Pops up a file chooser to allow the user to select the destination directory.
     * @param evt callback event
     */    
    private void saveHierMappingButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveHierMappingButtActionPerformed
        // Add your handling code here:
        JFileChooser chooser = new JFileChooser(Utils.lastDir);
        chooser.setDialogTitle("Select Directory to save into");
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
           System.out.println("You chose to save this file: " +
            chooser.getSelectedFile().getName());
           Utils.lastDir=chooser.getCurrentDirectory(); // save for next time
           try{
               processor.saveMappingHier(chooser.getSelectedFile());
           } catch (Exception e){
                ErrorPopup ep=new ErrorPopup(this, true);  
                ep.popup("Unable to write mapping hierarchy [" + 
                          chooser.getSelectedFile().getName() +
                           "due to exception = " + e.toString());
                return;
           } 
        }
    }//GEN-LAST:event_saveHierMappingButtActionPerformed

    /** Callback when "Edit Existing" mapping button is pressed.
     * This method gets the existing mapping for a table or class
     * (depending upon which is highlighted [class overrides table]
     * and sends them to the detail mapping form.
     * @param evt callback event
     */    
    private void editExistButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editExistButtActionPerformed
        // Add your handling code here:
        if ( classList.isSelectionEmpty() ) return;
        String classname=(String)classList.getSelectedValue();
        ClassTableMap map=processor.getMappedEntry(classname);
        if ( map == null ){
            System.out.println("Internal error, couldn't find class " + classname + "as mapped");
            return;
        }
        MappingDetails md=new MappingDetails(this,false);
        md.mapInfo(processor,map);
        md.show();
        refresh();
    }//GEN-LAST:event_editExistButtActionPerformed

    /** Callback for the combo box associated with the
     * class list. Calls the processor to set the
     * appropriate state flag for future calls to retrieve
     * the class list.
     * @param evt callback event
     */    
    private void dbtabModComboActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dbtabModComboActionPerformed
        // Add your handling code here:
        String tableSelectType=(String)dbtabModCombo.getSelectedItem();
        if ( tableSelectType.equals("All") ){
            processor.setDbTabMode(SourceProcessor.TABLE_SELECTION_ALL);
        }else if ( tableSelectType.equals("Unmapped") ){
            processor.setDbTabMode(SourceProcessor.TABLE_SELECTION_UNMAPPED);
        }else if ( tableSelectType.equals("Mapped") ){
            processor.setDbTabMode(SourceProcessor.TABLE_SELECTION_MAPPED);
        }
        refresh();
    }//GEN-LAST:event_dbtabModComboActionPerformed

    /** Callback for the combo box associated with the
     * table list. Calls the processor to set the
     * appropriate state flag for future calls to retrieve
     * the table list.
     * @param evt callback event
     */    
    private void classModComboActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_classModComboActionPerformed
        // Add your handling code here:
        String classSelectType=(String)classModCombo.getSelectedItem();
        if ( classSelectType.equals("All") ){
            processor.setClassMode(SourceProcessor.CLASS_SELECTION_ALL);
        }else if ( classSelectType.equals("Unmapped") ){
            processor.setClassMode(SourceProcessor.CLASS_SELECTION_UNMAPPED);
        }else if ( classSelectType.equals("Mapped") ){
            processor.setClassMode(SourceProcessor.CLASS_SELECTION_MAPPED);
        }
        refresh();
    }//GEN-LAST:event_classModComboActionPerformed

    /** Callback method attached to the "Load state" button.
     * This method pops up a file chooser to allow the user
     * to select a file to load then it attempts to load
     * that file.
     * @param evt callback event
     */    
    private void loadStateButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadStateButtActionPerformed
        // Add your handling code here:
        JFileChooser chooser = new JFileChooser(Utils.lastDir);
        chooser.setDialogTitle("Select state file to load:");
        ExampleFileFilter filter = new ExampleFileFilter();
        filter.addExtension("jof");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
           System.out.println("You chose to load this file: " +
            chooser.getSelectedFile().getName());
           Utils.lastDir=chooser.getCurrentDirectory(); // save for next time
           try {
	       ObjectInputStream ois = new ObjectInputStream(
                                          new FileInputStream(chooser.getSelectedFile()));
               processor=(SourceProcessor)ois.readObject();
               ois.close();
           }catch (Exception e){
               System.out.println("Unable to read state, sorry. " + e);
               e.printStackTrace();
           }
        }
        refresh();
    }//GEN-LAST:event_loadStateButtActionPerformed

    /** "Save state" callback. Pop up a file chooser, then write the
     * state to the selected file
     * @param evt callback event
     */    
    private void saveStateButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveStateButtActionPerformed
        // Add your handling code here:
        JFileChooser chooser = new JFileChooser(Utils.lastDir);
        chooser.setDialogTitle("Select file to save state into:");
        ExampleFileFilter filter = new ExampleFileFilter();
        filter.addExtension("jof");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
           System.out.println("You chose to load this file: " +
            chooser.getSelectedFile().getName());
           Utils.lastDir=chooser.getCurrentDirectory(); // save for next time
           try {
	       ObjectOutputStream oos = new ObjectOutputStream(
                                          new FileOutputStream(chooser.getSelectedFile()));
               oos.writeObject(processor);
               oos.close();
           }catch (Exception e){
               System.out.println("Unable to write state, sorry. " + e);
               e.printStackTrace();
           }
        }
    }//GEN-LAST:event_saveStateButtActionPerformed

    /** "Show Mapping" button callback. Pop up a text viewer
     * with the xml code for the current mappings
     * @param evt callback event
     */    
    private void showMappingButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showMappingButtActionPerformed
        // Add your handling code here:
        new TextViewer(this,true,processor.getMappingSource()).show();
    }//GEN-LAST:event_showMappingButtActionPerformed

    /** "Config" button callback. Pop up the configuration dialog.
     * @param evt callback event
     */    
    private void configButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_configButtActionPerformed
        DbMapperConfig configForm=new DbMapperConfig(this, true);
        configForm.show();
        Config conf=configForm.getConfig();
        configForm.dispose();
        if ( conf == null ) return;
        try {
            processor.load(conf);
        } catch (ConfigException ce){
            ErrorPopup ep=new ErrorPopup(this, true);  
            ep.popup("Unable to load mapping info. Exception: " + ce.toString());
            return;
        }
    }//GEN-LAST:event_configButtActionPerformed

    /** Callback when "Quit" button is called. Just exit.
     * @param evt callback event
     */    
    private void quitButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_quitButtActionPerformed
        // Add your handling code here:
        System.exit(0);
    }//GEN-LAST:event_quitButtActionPerformed

    /** Callback when "Save mapping file" button is called. Pop up a file
     * chooser dialog, then save the xml mapping file to the
     * user selected file.
     * @param evt callback event
     */    
    private void saveMappingButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveMappingButtActionPerformed
        // Add your handling code here:
        JFileChooser chooser = new JFileChooser(Utils.lastDir);
        chooser.setDialogTitle("Select file to save maping into:");
        ExampleFileFilter filter = new ExampleFileFilter();
        filter.addExtension("xml");
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
           System.out.println("You chose to save this file: " +
            chooser.getSelectedFile().getName());
           Utils.lastDir=chooser.getCurrentDirectory(); // save for next time
           try{
               String code=processor.getMappingSource();
               FileOutputStream out=new FileOutputStream(chooser.getSelectedFile());
               out.write(code.getBytes());
               out.close();
           } catch (Exception e){
                ErrorPopup ep=new ErrorPopup(this, true);  
                ep.popup("Unable to write file [" + chooser.getSelectedFile().getName() +
                           "due to exception = " + e.toString());
                return;
           } 
        }
    }//GEN-LAST:event_saveMappingButtActionPerformed

    /** "Map selected" button callback.<p>
     * If a class AND a table are selected. get their names and sned them to
     * a new MapDetail form.
     * @param evt callback event
     */    
    private void mapSelectedButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapSelectedButtActionPerformed
        // Add your handling code here:
        if ( classList.isSelectionEmpty() ||
             tableList.isSelectionEmpty() ) return;
        String classname=(String)classList.getSelectedValue();
        String tablename=(String)tableList.getSelectedValue();
        MappingDetails md=new MappingDetails(this,false);
        md.mapInfo(processor,classname,tablename);
        md.show();
        processor.setClassInUse(classname);
        processor.setTableInUse(tablename);
        refresh();
    }//GEN-LAST:event_mapSelectedButtActionPerformed
    
    /** Exit the Application - bye bye
     * @param evt event
     */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton saveHierMappingButt;
    private javax.swing.JButton showMappingButt;
    private javax.swing.JLabel showLab1;
    private javax.swing.JButton mapSelectedButt;
    private javax.swing.JLabel showLab2;
    private javax.swing.JScrollPane tableScrollerPane;
    private javax.swing.JButton loadDbCl;
    private javax.swing.JButton configButt;
    private javax.swing.JButton saveMappingButt;
    private javax.swing.JList tableList;
    private javax.swing.JButton loadStateButt;
    private javax.swing.JList classList;
    private javax.swing.JButton quitButt;
    private javax.swing.JComboBox dbtabModCombo;
    private javax.swing.JLabel classLab;
    private javax.swing.JButton editExistButt;
    private javax.swing.JLabel dbtabLab;
    private javax.swing.JComboBox classModCombo;
    private javax.swing.JLabel assocChooTitLab;
    private javax.swing.JScrollPane classScrollPane;
    private javax.swing.JButton saveStateButt;
    // End of variables declaration//GEN-END:variables
    
}
