/*
 * MappedCellRenderer.java
 *
 * A cell renderer which will display the cell in different colors 
 * depending on the state of the item.
 *
 * Under development
 *
 * Created on April 21, 2003, 1:44 PM
 */

package dbmapper;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.UIManager;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Insets;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;


/**
 *
 * @author  Larry
 */
public class MappedCellRenderer extends JLabel implements ListCellRenderer {
  
    public static final Color UNMAPPED = Color.red;
    public static final Color MAPPED_FULL = Color.green;
    public static final Color MAPPED_PART = Color.orange;
    
    SourceProcessor myProc=null;
    String myType = null;
    
    protected static Border m_noFocusBorder;
    protected FontMetrics m_fm = null;
    protected Insets m_insets = new Insets(0, 0, 0, 0);
    
    /** Creates a new instance of MappedCellRenderer */
    public MappedCellRenderer(SourceProcessor proc, String type) {
        super();
        myProc=proc;
        myType=type;
        m_noFocusBorder = new EmptyBorder(1, 1, 1, 1);
        setOpaque(true);
	setBorder(m_noFocusBorder);
    }


    public Component getListCellRendererComponent(JList list,
		Object value, int index, boolean isSelected, boolean cellHasFocus)     
    {         
	setText(value.toString());

	setBackground(isSelected ? list.getSelectionBackground() : list.getBackground());
	setForeground(isSelected ? list.getSelectionForeground() : list.getForeground());
		
	setFont(list.getFont());
	setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : m_noFocusBorder);

	return this;
    }


    public void paint(Graphics g)
    {
	m_fm = g.getFontMetrics();
	
	g.setColor(getBackground());
	g.fillRect(0, 0, getWidth(), getHeight());
	getBorder().paintBorder(this, g, 0, 0, getWidth(), getHeight());

	g.setColor(getForeground());
	g.setFont(getFont());
	m_insets = getInsets();
	int x = m_insets.left;
	int y = m_insets.top + m_fm.getAscent();

	g.drawString(getText(), x, y);
    }

}
