/*
 * JDialog.java
 *
 * Created on April 10, 2003, 1:41 PM
 */

package dbmapper;

import java.util.HashMap;
import javax.swing.DefaultListModel;
import java.util.Set;
import java.util.Iterator;

/** This class creates the Mapping Details dialog for mapping class atttributes
 * to table columns.<p>
 * @author Larry
 */
public class MappingDetails extends javax.swing.JDialog {
    
    /** The parent frame we need to pop up dialogs
     */    
    java.awt.Frame myParentFrame=null;
    /** My processor
     */    
    SourceProcessor processor=null;
    /** The classname this is mapping
     */    
    String classname=null;
    /** The tablename this is mapping
     */    
    String tablename=null;
    
    /** The list model for the attributes
     */    
    DefaultListModel attrListModel = null;
    /** The list model for the db columns
     */    
    DefaultListModel dbcolListModel = null;
    /** The list model for the class extends list
     */    
    DefaultListModel extListModel = null;
    /** The list model for the mapped pairings
     */    
    DefaultListModel mappedListModel = null;
    
    /** The ClassTableMap defining the class to table map this form is
     * working on.
     */    
    ClassTableMap myMap = null;
    
    /** Max number of properties (should be loaded from a real property file)
     */    
    private static final int MAXPROPS=25;
    /** Cache of Textfields used to display the property keys
     */    
    javax.swing.JTextField[] prop=new javax.swing.JTextField[MAXPROPS];
    /** Cache of Textfields used to display the property values
     */    
    javax.swing.JTextField[] vals=new javax.swing.JTextField[MAXPROPS];
    /** The current selected map pair in the mapping list
     */    
    private int currSelectedMapIndex=-1;
    
    /** Creates new form MappingDetails
     * @param parent parent frame
     * @param modal t/f/ modal flag
     */
    public MappingDetails(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
        initProps();
        myParentFrame=parent;
    }
    
    /** set the mapping data sources for this mapping dialog
     * @param proc the source processor to use
     * @param map the map to save the mapping details to
     */    
    public void mapInfo(SourceProcessor proc,ClassTableMap map){
        processor=proc;
        myMap=map;
        mapSetup();
    }
    
    /** set the mapping data source for this mapping dialog and create a new
     * ClassTableMap to store the mapping information.
     * @param proc The processor to use
     * @param cname The classname to map
     * @param tname The tablename to map
     */    
    public void mapInfo(SourceProcessor proc, String cname, String tname){
        processor=proc;
        myMap=new ClassTableMap(processor, cname, tname);
        mapSetup();
    }
        
    /** Set up the initial mapping information. Get the list of class attributes,
     * get the list of database columns, get the list of class extenders.<p>
     * Create new ListModels, Populate using allListUpate method, then assign
     * filled models to the Jlist's.
     */    
    private void mapSetup(){
        classname=myMap.getClassname();
        tablename=myMap.getTablename();
        classNameField.setText(classname);
        dbTableNameField.setText(tablename);
        
        attrListModel = new DefaultListModel();
        dbcolListModel = new DefaultListModel();
        extListModel = new DefaultListModel();
        mappedListModel = new DefaultListModel();
        allListUpdate();
        
        classAttList.setModel(attrListModel);
        dbColList.setModel(dbcolListModel);
        extendList.setModel(extListModel);
        mappedList.setModel(mappedListModel);
    }
    
    /** Clear the list models, get fresh lists from the map class and update the
     * mappedList using mappedListUpdate method.
     */    
    private void allListUpdate(){
        attrListModel.clear();
        Object[] attrs=myMap.getUnmappedAttrs();
        for (int i=0; i<attrs.length; i++){
            attrListModel.addElement(attrs[i]);
        }
        dbcolListModel.clear();
        Object[] cols=myMap.getUnmappedCols();
        for (int i=0; i<cols.length; i++){
            dbcolListModel.addElement(cols[i]);
        }
        extListModel.clear();
        Object[] ext=myMap.getExtList();
        for (int i=0; i<ext.length; i++){
            extListModel.addElement(ext[i]);
        }
        mappedListUpdate();
    }
   
    /** Update the mapped list using information in the map. Refresh the list
     * display, select the last list item if it has any, and display it's
     * properties
     */    
    private void mappedListUpdate(){
        mappedListModel.clear();
        String [] desc=myMap.getMappedDescrition();
        for (int i=0; i<desc.length; i++ ){
            mappedListModel.addElement(desc[i]);
        }
        MappingProperties mp=null;
        int lastix=desc.length-1;
        //System.out.println("Last viewable index = " + lastix);
        if ( lastix >= 0 ){
           mappedList.setSelectedIndex(lastix);
           mappedList.ensureIndexIsVisible(lastix);
           mp=myMap.getMappedProperties(lastix);
           currSelectedMapIndex=lastix;
        }
        propRefresh(mp);
        mappedList.repaint();
    }
  
    /** create the cached textfields for the property panel display
     */    
    private void initProps(){
        propPan.removeAll();
        for (int i=0; i<MAXPROPS; i++){
            prop[i]=new javax.swing.JTextField();
            prop[i].setVisible(false);
            prop[i].setEditable(false);
            prop[i].setText("                  "); //18 spaces
            vals[i]=new javax.swing.JTextField();
            vals[i].setVisible(false);
            vals[i].setText("       "); //7 s
            propPan.add(prop[i]);
            propPan.add(vals[i]);
        }
        pack();
    }
            
    /** clear the properties panel textfields
     */    
    private void clearProps(){
        for (int i=0; i<MAXPROPS; i++){
            prop[i].setText("");
            prop[i].setVisible(false);
            vals[i].setText("");
            vals[i].setVisible(false);
        }
    }
    
    /** Save the current property values for the map pair at the position
     * specified by the index.
     * @param index The mapping pair that the properties apply to.
     */    
    private void saveProps(int index){
        MappingProperties prevMp = myMap.getMappedProperties(index);
        for ( int i=0; i<MAXPROPS; i++ ){
            if ( ! prop[i].isVisible() ) continue;
            String value=(String)prevMp.get(prop[i].getText());
            if ( value== null || value.equals(vals[i].getText())) continue; //nochange
            prevMp.put(prop[i].getText(), vals[i].getText());
        }
    }
    
    /** Set the property panel to display the properties
     * as specified in the passed in MappingProperties object
     * @param mp properties to display
     */    
    private void propRefresh(MappingProperties mp){
        clearProps();
        if ( mp == null ) return;
        Set keys=mp.keySet();
        Iterator keyiter=keys.iterator();
        int i=0;
        while ( keyiter.hasNext() ){
            String propname=(String)keyiter.next();
            String value=(String)mp.get(propname);
            prop[i].setText(propname);
            prop[i].setVisible(true);
            vals[i].setText(value);
            vals[i].setVisible(true);
            i++;
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        mapdetTitLab = new javax.swing.JLabel();
        classTitLab = new javax.swing.JLabel();
        classNameField = new javax.swing.JTextField();
        sbtabTitLab = new javax.swing.JLabel();
        dbTableNameField = new javax.swing.JTextField();
        mappedPanel = new javax.swing.JScrollPane();
        mappedList = new javax.swing.JList();
        classAttLab = new javax.swing.JLabel();
        dbColsLab = new javax.swing.JLabel();
        dbColsPanel = new javax.swing.JScrollPane();
        dbColList = new javax.swing.JList();
        classAttPanel = new javax.swing.JScrollPane();
        classAttList = new javax.swing.JList();
        mapPKButt = new javax.swing.JButton();
        mapPrimitivButt = new javax.swing.JButton();
        mapObjRefButt = new javax.swing.JButton();
        mapCollectionButt = new javax.swing.JButton();
        attLab = new javax.swing.JLabel();
        colLab = new javax.swing.JLabel();
        okButt = new javax.swing.JButton();
        cancelButt = new javax.swing.JButton();
        viewSourceButt = new javax.swing.JButton();
        extendLab = new javax.swing.JLabel();
        extendListPan = new javax.swing.JScrollPane();
        extendList = new javax.swing.JList();
        addExtButt = new javax.swing.JButton();
        notMappedButton = new javax.swing.JButton();
        manymanyButton = new javax.swing.JButton();
        moveUpButt = new javax.swing.JButton();
        moveDownButt = new javax.swing.JButton();
        unMapButt = new javax.swing.JButton();
        propScrollPan = new javax.swing.JScrollPane();
        propPan = new javax.swing.JPanel();
        jTextField1 = new javax.swing.JTextField();
        jTextField2 = new javax.swing.JTextField();
        jTextField3 = new javax.swing.JTextField();
        jTextField4 = new javax.swing.JTextField();
        jTextField5 = new javax.swing.JTextField();
        jTextField6 = new javax.swing.JTextField();
        jTextField7 = new javax.swing.JTextField();
        jTextField8 = new javax.swing.JTextField();
        jTextField9 = new javax.swing.JTextField();
        jTextField10 = new javax.swing.JTextField();
        jTextField11 = new javax.swing.JTextField();
        jTextField12 = new javax.swing.JTextField();
        jTextField13 = new javax.swing.JTextField();
        jTextField14 = new javax.swing.JTextField();
        jTextField15 = new javax.swing.JTextField();
        jTextField16 = new javax.swing.JTextField();
        jTextField17 = new javax.swing.JTextField();
        jTextField18 = new javax.swing.JTextField();
        jTextField19 = new javax.swing.JTextField();
        jTextField20 = new javax.swing.JTextField();
        delExtButt = new javax.swing.JButton();

        getContentPane().setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        mapdetTitLab.setFont(new java.awt.Font("Century Schoolbook", 1, 14));
        mapdetTitLab.setText("Mapping Details");
        getContentPane().add(mapdetTitLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(300, 10, -1, -1));

        classTitLab.setText("Class:");
        getContentPane().add(classTitLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 40, -1, -1));

        classNameField.setEditable(false);
        classNameField.setText("class");
        getContentPane().add(classNameField, new org.netbeans.lib.awtextra.AbsoluteConstraints(70, 40, 320, -1));

        sbtabTitLab.setText("DB Table:");
        getContentPane().add(sbtabTitLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(500, 40, 60, -1));

        dbTableNameField.setEditable(false);
        dbTableNameField.setText("table");
        getContentPane().add(dbTableNameField, new org.netbeans.lib.awtextra.AbsoluteConstraints(560, 40, 220, -1));

        mappedList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                mappedListMouseClicked(evt);
            }
        });

        mappedPanel.setViewportView(mappedList);

        getContentPane().add(mappedPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 100, 550, 140));

        classAttLab.setText("Class Attributes:");
        getContentPane().add(classAttLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 320, -1, -1));

        dbColsLab.setText("DB Columns:");
        getContentPane().add(dbColsLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(450, 320, -1, -1));

        dbColList.setBorder(new javax.swing.border.EtchedBorder());
        dbColList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "1", "2", "3", "4", "5", "6", "7", "8" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        dbColList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        dbColsPanel.setViewportView(dbColList);

        getContentPane().add(dbColsPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(450, 340, 250, 170));

        classAttList.setBorder(new javax.swing.border.EtchedBorder());
        classAttList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "A", "B", "C", "D", "E", "F", "G", "H" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        classAttList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        classAttPanel.setViewportView(classAttList);

        getContentPane().add(classAttPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 340, 280, 170));

        mapPKButt.setText(" Map as PK ");
        mapPKButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mapPKButtActionPerformed(evt);
            }
        });

        getContentPane().add(mapPKButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 340, -1, -1));

        mapPrimitivButt.setText("Map as Primitive");
        mapPrimitivButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mapPrimitivButtActionPerformed(evt);
            }
        });

        getContentPane().add(mapPrimitivButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 370, -1, -1));

        mapObjRefButt.setText("Map as ObjRef");
        mapObjRefButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mapObjRefButtActionPerformed(evt);
            }
        });

        getContentPane().add(mapObjRefButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 400, -1, -1));

        mapCollectionButt.setText("Map as Collection");
        mapCollectionButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mapCollectionButtActionPerformed(evt);
            }
        });

        getContentPane().add(mapCollectionButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 430, -1, -1));

        attLab.setFont(new java.awt.Font("Century Schoolbook", 0, 10));
        attLab.setText("Attribute");
        getContentPane().add(attLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(70, 80, -1, -1));

        colLab.setFont(new java.awt.Font("Century Schoolbook", 0, 10));
        colLab.setText("Column");
        getContentPane().add(colLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(420, 80, -1, -1));

        okButt.setText("Ok");
        okButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtActionPerformed(evt);
            }
        });

        getContentPane().add(okButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 530, -1, -1));

        cancelButt.setText("Cancel");
        cancelButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtActionPerformed(evt);
            }
        });

        getContentPane().add(cancelButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(390, 530, -1, -1));

        viewSourceButt.setText("ViewSource");
        viewSourceButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                viewSourceButtActionPerformed(evt);
            }
        });

        getContentPane().add(viewSourceButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(600, 530, -1, -1));

        extendLab.setText("extended by:");
        getContentPane().add(extendLab, new org.netbeans.lib.awtextra.AbsoluteConstraints(40, 250, -1, -1));

        extendList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "A1", "A2", "A3", "A4", "A5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        extendListPan.setViewportView(extendList);

        getContentPane().add(extendListPan, new org.netbeans.lib.awtextra.AbsoluteConstraints(140, 250, 270, 60));

        addExtButt.setText("Add...");
        addExtButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addExtButtActionPerformed(evt);
            }
        });

        getContentPane().add(addExtButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(420, 250, -1, -1));

        notMappedButton.setText("Mark Not Mapped");
        notMappedButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                notMappedButtonActionPerformed(evt);
            }
        });

        getContentPane().add(notMappedButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 490, -1, -1));

        manymanyButton.setText("Mark Many-Many");
        manymanyButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                manymanyButtonActionPerformed(evt);
            }
        });

        getContentPane().add(manymanyButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 460, -1, -1));

        moveUpButt.setText("^");
        moveUpButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveUpButtActionPerformed(evt);
            }
        });

        getContentPane().add(moveUpButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(570, 110, -1, -1));

        moveDownButt.setText("v");
        moveDownButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveDownButtActionPerformed(evt);
            }
        });

        getContentPane().add(moveDownButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(570, 210, -1, -1));

        unMapButt.setText("X");
        unMapButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                unMapButtActionPerformed(evt);
            }
        });

        getContentPane().add(unMapButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(570, 160, -1, -1));

        propPan.setLayout(new java.awt.GridLayout(0, 2));

        jTextField1.setText("jTextField1");
        propPan.add(jTextField1);

        jTextField2.setText("jTextField2");
        propPan.add(jTextField2);

        jTextField3.setText("jTextField3");
        propPan.add(jTextField3);

        jTextField4.setText("jTextField4");
        propPan.add(jTextField4);

        jTextField5.setText("jTextField5");
        propPan.add(jTextField5);

        jTextField6.setText("jTextField6");
        propPan.add(jTextField6);

        jTextField7.setText("jTextField7");
        propPan.add(jTextField7);

        jTextField8.setText("jTextField8");
        propPan.add(jTextField8);

        jTextField9.setText("jTextField9");
        propPan.add(jTextField9);

        jTextField10.setText("jTextField10");
        propPan.add(jTextField10);

        jTextField11.setText("jTextField11");
        propPan.add(jTextField11);

        jTextField12.setText("jTextField12");
        propPan.add(jTextField12);

        jTextField13.setText("jTextField13");
        propPan.add(jTextField13);

        jTextField14.setText("jTextField14");
        propPan.add(jTextField14);

        jTextField15.setText("jTextField15");
        propPan.add(jTextField15);

        jTextField16.setText("jTextField16");
        propPan.add(jTextField16);

        jTextField17.setText("jTextField17");
        propPan.add(jTextField17);

        jTextField18.setText("jTextField18");
        propPan.add(jTextField18);

        jTextField19.setText("jTextField19");
        propPan.add(jTextField19);

        jTextField20.setText("jTextField20");
        propPan.add(jTextField20);

        propScrollPan.setViewportView(propPan);

        getContentPane().add(propScrollPan, new org.netbeans.lib.awtextra.AbsoluteConstraints(613, 93, 200, 220));

        delExtButt.setText("Remove");
        delExtButt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                delExtButtActionPerformed(evt);
            }
        });

        getContentPane().add(delExtButt, new org.netbeans.lib.awtextra.AbsoluteConstraints(420, 280, -1, -1));

        pack();
    }//GEN-END:initComponents

    /** "Delete" extent callback button. Remove the selected class
     * from the list of extending classes.
     *
     *
     * @param evt callback event
     */    
    private void delExtButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_delExtButtActionPerformed
        // Add your handling code here:
        if ( extendList.isSelectionEmpty() ) return;
        myMap.delExt((String)extendList.getSelectedValue());
        this.allListUpdate();
    }//GEN-LAST:event_delExtButtActionPerformed

    /** "Mouse-Click" in Mapped list callback button. Should mean
     * the user has selected another map pair to display. So, make sure we
     * save any property changes they may have made, then display the new
     * properties for the map pair they selected.
     * @param evt callback event
     */    
    private void mappedListMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mappedListMouseClicked
        // Add your handling code here:
        if ( mappedList.isSelectionEmpty() ) return;
        if ( currSelectedMapIndex >= 0 ) saveProps(currSelectedMapIndex);
        MappingProperties mp=myMap.getMappedProperties(mappedList.getSelectedIndex());
        propRefresh(mp);
        currSelectedMapIndex=mappedList.getSelectedIndex();
    }//GEN-LAST:event_mappedListMouseClicked

    /** "Unmap" button callback. Unmap the selected map pair from the
     * map list of mapped pairs. Return the attribute and column to the list
     * of unmapped attributes/columns.
     *
     * @param evt callback event
     */    
    private void unMapButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_unMapButtActionPerformed
        // Add your handling code here:
        // unmap selected item
        if ( mappedList.isSelectionEmpty() ) return;
        myMap.unMap(mappedList.getSelectedIndex());
        this.allListUpdate();
    }//GEN-LAST:event_unMapButtActionPerformed

    /** "Move down" button callback. Move the selected map pair down one slot
     * in the map list.
     * @param evt callback event
     */    
    private void moveDownButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moveDownButtActionPerformed
        // Add your handling code here:
        if ( mappedList.isSelectionEmpty() ) return;
        int selectedItem = mappedList.getSelectedIndex();
        saveProps(selectedItem);
        myMap.moveMappedDown(selectedItem);
        this.mappedListUpdate();
    }//GEN-LAST:event_moveDownButtActionPerformed

    /** "Move down" button callback. Move the selected map pair up one slot
     * in the map list.
     * @param evt callback event
     */    
    private void moveUpButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moveUpButtActionPerformed
        // Add your handling code here:
        if ( mappedList.isSelectionEmpty() ) return;
        int selectedItem = mappedList.getSelectedIndex();
        saveProps(selectedItem);
        myMap.moveMappedUp(selectedItem);
        this.mappedListUpdate();
    }//GEN-LAST:event_moveUpButtActionPerformed

    /** "Make ManyMany" callback button. This is used to mark an attribute as a
     * many:many reference for a collection. First pop up a dialog to get the
     * mapping table, FK and remote FK, then call the makeMany of the map object
     * to create the map.
     * @param evt callback event
     */    
    private void manymanyButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_manymanyButtonActionPerformed
        // Add your handling code here:
        // Add your handling code here:
        if ( classAttList.isSelectionEmpty() ) return;
        String attrname=(String)classAttList.getSelectedValue();
        ManyMappSelector mms=new ManyMappSelector(myParentFrame,true);
        mms.setClassname(classname);
        mms.setAttr(attrname);
        mms.setProcessor(processor);
        mms.setTableList(processor.getTableList());
        ManyMap mm=new ManyMap();
        mms.setResults(mm);
        mms.show();
        while ( mms.isVisible() ){
            try{
                Thread.sleep(200);
            }catch (Exception e){}
        }
        if ( mm.isValid() ){
            myMap.makeManyref(attrname, mm);
            this.mappedListUpdate();
            attrListModel.removeElementAt(classAttList.getSelectedIndex());
            repaint();
        }
    }//GEN-LAST:event_manymanyButtonActionPerformed

    /** "View Source" callback button. Popup a viewer window showing the
     * xml mapping text for the maps created so far.
     * @param evt callback event
     */    
    private void viewSourceButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_viewSourceButtActionPerformed
        new TextViewer(myParentFrame,true,myMap.getMapSource()).show();
    }//GEN-LAST:event_viewSourceButtActionPerformed

    /** "Add" extent callback button. Popup a user input dialog to get the
     * name of a class to add to the extends list.
     * @param evt callback event
     */    
    private void addExtButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addExtButtActionPerformed
        UserInputDialog inDia=new UserInputDialog(myParentFrame, true, "Enter classname to add");
        inDia.show();
        String newClass=inDia.getInput();
        inDia.dispose();
        myMap.addExt(newClass);
        this.allListUpdate();
    }//GEN-LAST:event_addExtButtActionPerformed

    /** "Ok" callback button. Send the map to the processor for inclusion
     * into the "finished" map. hide this dialog.
     * @param evt callback event
     */    
    private void okButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtActionPerformed
        processor.addMapping((MappingDescriptor)myMap);
        setVisible(false);
        dispose();
    }//GEN-LAST:event_okButtActionPerformed

    /** "Not mapped" callback button. This is used to mark an attribute or
     * column as not used. Check which is selected (if both, just ignore the column)
     * then call the not mapped method in the map to set it as not mapped.
     * @param evt callback event
     */    
    private void notMappedButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_notMappedButtonActionPerformed
        if ( ! classAttList.isSelectionEmpty() ) {
            String attrname=(String)classAttList.getSelectedValue();
            myMap.makeDudClassref(attrname);
            int atix=classAttList.getSelectedIndex();
            attrListModel.removeElementAt(atix);
        }else if ( ! dbColList.isSelectionEmpty() ) {
            String colname=(String)dbColList.getSelectedValue();
            myMap.makeDudDbcolref(colname);
            int clix=dbColList.getSelectedIndex();
            dbcolListModel.removeElementAt(clix);
        }
        this.mappedListUpdate();
        repaint();
        return;
    }//GEN-LAST:event_notMappedButtonActionPerformed

    /** "Make Collection" callback button. This is used to mark an attribute as a
     * 1:many reference for a collection. First pop up a dialog to get the
     * remote class and FK, then call the makeColl of the map object to create
     * the map.
     * @param evt callback event
     */    
    private void mapCollectionButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapCollectionButtActionPerformed
        if ( classAttList.isSelectionEmpty() ) return;
        String attrname=(String)classAttList.getSelectedValue();
        CollMappSelector mms=new CollMappSelector(myParentFrame,true);
        mms.setClassname(classname);
        mms.setAttr(attrname);
        mms.setProcessor(processor);
        mms.setClassList(processor.getClassList(SourceProcessor.CLASS_SELECTION_ALL));
        CollMap cm=new CollMap();
        mms.setResults(cm);
        mms.show();
        while ( mms.isVisible() ){
            try{
                Thread.sleep(200);
            }catch (Exception e){}
        }
        if ( cm.isValid() ){
            myMap.makeCollref(attrname, cm);
            this.mappedListUpdate();
            attrListModel.removeElementAt(classAttList.getSelectedIndex());
            repaint();
        }
    }//GEN-LAST:event_mapCollectionButtActionPerformed

    /** "Make ObjRef" callback button. This is used to mark an attribute and
     * column pair as an object ref pair. First make sure we have an attribute
     * and a column selected, then call the makeObjRef method in the map to set these as
     * an object ref pair.
     * @param evt callback event
     */    
    private void mapObjRefButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapObjRefButtActionPerformed
        // Add your handling code here:
        if ( classAttList.isSelectionEmpty() ||
             dbColList.isSelectionEmpty() ) return;
        String attrname=(String)classAttList.getSelectedValue();
        String colname=(String)dbColList.getSelectedValue();
        myMap.makeObjref(attrname, colname);
        this.mappedListUpdate();
        attrListModel.removeElementAt(classAttList.getSelectedIndex());
        dbcolListModel.removeElementAt(dbColList.getSelectedIndex());
        repaint();
    }//GEN-LAST:event_mapObjRefButtActionPerformed

    /** "Make Prim" callback button. This is used to mark an attribute and
     * column pair as a primitive type pair. First make sure we have an attribute
     * and a column selected, then call the makePrim method in the map to set these as
     * primitive map pair.
     * @param evt callback event
     */    
    private void mapPrimitivButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapPrimitivButtActionPerformed
        // Add your handling code here:
        if ( classAttList.isSelectionEmpty() ||
             dbColList.isSelectionEmpty() ) return;
        String attrname=(String)classAttList.getSelectedValue();
        String colname=(String)dbColList.getSelectedValue();
        myMap.makePrimref(attrname, colname);
        this.mappedListUpdate();
        attrListModel.removeElementAt(classAttList.getSelectedIndex());
        dbcolListModel.removeElementAt(dbColList.getSelectedIndex());
        repaint();
    }//GEN-LAST:event_mapPrimitivButtActionPerformed

    /** "Make PK" callback button. This is used to mark an attribute and
     * column pair as the Primark key. First make sure we have an attribute
     * and a column selected, then call the makePk method in the map to set these as
     * the PK map pair.
     * @param evt callback event
     */    
    private void mapPKButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapPKButtActionPerformed
        // Add your handling code here:
        if ( classAttList.isSelectionEmpty() ||
             dbColList.isSelectionEmpty() ) return;
        String attrname=(String)classAttList.getSelectedValue();
        String colname=(String)dbColList.getSelectedValue();
        myMap.makePKref(attrname, colname);
        this.mappedListUpdate();
        attrListModel.removeElementAt(classAttList.getSelectedIndex());
        dbcolListModel.removeElementAt(dbColList.getSelectedIndex());
        repaint();
    }//GEN-LAST:event_mapPKButtActionPerformed

    /** drop any maps we made and just quit.
     */    
    private void goaway(){
        processor.unsetClassInUse(classname);
        processor.unsetTableInUse(tablename);
        setVisible(false);
        dispose();
    }
    
    /** "Cancel" callback button. Mark the map invalid and then hide this dialog.
     * @param evt callback event
     */    
    private void cancelButtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtActionPerformed
        goaway();
    }//GEN-LAST:event_cancelButtActionPerformed
    
    /** Closes the dialog
     * @param evt callback event
     */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        goaway();
    }//GEN-LAST:event_closeDialog
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField jTextField6;
    private javax.swing.JButton unMapButt;
    private javax.swing.JTextField jTextField7;
    private javax.swing.JTextField jTextField3;
    private javax.swing.JTextField jTextField12;
    private javax.swing.JButton mapObjRefButt;
    private javax.swing.JTextField jTextField8;
    private javax.swing.JTextField jTextField19;
    private javax.swing.JScrollPane dbColsPanel;
    private javax.swing.JButton mapCollectionButt;
    private javax.swing.JTextField jTextField5;
    private javax.swing.JButton okButt;
    private javax.swing.JList mappedList;
    private javax.swing.JTextField jTextField13;
    private javax.swing.JLabel classAttLab;
    private javax.swing.JTextField jTextField10;
    private javax.swing.JList classAttList;
    private javax.swing.JButton cancelButt;
    private javax.swing.JLabel classTitLab;
    private javax.swing.JTextField jTextField2;
    private javax.swing.JTextField classNameField;
    private javax.swing.JLabel sbtabTitLab;
    private javax.swing.JTextField jTextField15;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JButton moveUpButt;
    private javax.swing.JPanel propPan;
    private javax.swing.JTextField jTextField14;
    private javax.swing.JLabel mapdetTitLab;
    private javax.swing.JTextField dbTableNameField;
    private javax.swing.JLabel dbColsLab;
    private javax.swing.JLabel attLab;
    private javax.swing.JScrollPane extendListPan;
    private javax.swing.JTextField jTextField9;
    private javax.swing.JButton notMappedButton;
    private javax.swing.JButton addExtButt;
    private javax.swing.JTextField jTextField17;
    private javax.swing.JScrollPane classAttPanel;
    private javax.swing.JTextField jTextField18;
    private javax.swing.JLabel colLab;
    private javax.swing.JTextField jTextField11;
    private javax.swing.JButton mapPrimitivButt;
    private javax.swing.JScrollPane propScrollPan;
    private javax.swing.JButton manymanyButton;
    private javax.swing.JButton moveDownButt;
    private javax.swing.JTextField jTextField16;
    private javax.swing.JScrollPane mappedPanel;
    private javax.swing.JList dbColList;
    private javax.swing.JButton viewSourceButt;
    private javax.swing.JList extendList;
    private javax.swing.JButton delExtButt;
    private javax.swing.JTextField jTextField4;
    private javax.swing.JLabel extendLab;
    private javax.swing.JTextField jTextField20;
    private javax.swing.JButton mapPKButt;
    // End of variables declaration//GEN-END:variables
    
}
