/*
 * MappingProperties.java
 *
 * Created on April 16, 2003, 3:10 PM
 */

package dbmapper;

import java.util.TreeMap;
/** This class encapsulates the properties for a mapping
 * between objects.
 * @author Larry
 */
public class MappingProperties extends TreeMap {
    
    /** Creates a new instance of MappingProperties */
    public MappingProperties() {
    }
    
    
    /** Get a new mapping property set
     * @param mapType The type of mapping link (PK, Primitive, etc)
     * @return A new MappingProperties map with the appropriate properties
     */    
    public static MappingProperties newMappingProperty(int mapType){
        switch (mapType){
            case ClassTableMap.MAP_TYPE_PK:{
                MappingProperties mp=new MappingProperties();
                mp.put("primarykey","true");
                mp.put("nullable","false");
                mp.put("indexed","false");
                mp.put("autoincrement","true");
                mp.put("sequence-name","");
                mp.put("locking","false");
                mp.put("default-fetch","true");
                mp.put("conversion","");
                mp.put("length","");
                mp.put("precision","");
                mp.put("scale","");
                mp.put("access","readwrite"); 
                return mp;
            }
            case ClassTableMap.MAP_TYPE_PRIM:{
                MappingProperties mp=new MappingProperties();
                mp.put("primarykey","false");
                mp.put("nullable","true");
                mp.put("indexed","false");
                mp.put("autoincrement","false");
                mp.put("sequence-name","");
                mp.put("locking","false");
                mp.put("default-fetch","true");
                mp.put("conversion","");
                mp.put("length","");
                mp.put("precision","");
                mp.put("scale","");
                mp.put("access","readwrite"); 
                return mp;
            }   
            case ClassTableMap.MAP_TYPE_OBJREF:{
                MappingProperties mp=new MappingProperties();
                mp.put("primarykey","false");
                mp.put("nullable","true");
                mp.put("indexed","false");
                mp.put("autoincrement","false");
                mp.put("sequence-name","");
                mp.put("locking","false");
                mp.put("default-fetch","true");
                mp.put("conversion","");
                mp.put("length","");
                mp.put("precision","");
                mp.put("scale","");
                mp.put("access","anonymous"); 
                //
                mp.put("RDproxy","false"); 
                mp.put("RDrefresh","false"); 
                mp.put("RDauto-retrieve ","true"); 
                mp.put("RDauto-update","false"); 
                mp.put("RDauto-delete","false"); 
                return mp;
            }   
            case ClassTableMap.MAP_TYPE_COLLECT:{
                MappingProperties mp=new MappingProperties();
                mp.put("orderby","");
                mp.put("sort","ASC");
                mp.put("indirection-table","");
                mp.put("proxy","false");
                mp.put("refresh","false");
                mp.put("auto-retrieve ","true"); 
                mp.put("auto-update","false"); 
                mp.put("auto-delete","false"); 
                return mp;
            }   
            case ClassTableMap.MAP_TYPE_MANY:{
                MappingProperties mp=new MappingProperties();
                mp.put("orderby","");
                mp.put("sort","ASC");
                mp.put("indirection-table","");
                mp.put("proxy","false");
                mp.put("refresh","false");
                mp.put("auto-retrieve ","true"); 
                mp.put("auto-update","false"); 
                mp.put("auto-delete","false"); 
                return mp;
            }    
            default:
                return null;
        }
    }
    
}
