/*
 * SourceProcessor.java
 *
 * Created on April 10, 2003, 7:12 PM
 */

package dbmapper;


import java.util.TreeMap;
import java.util.TreeSet;
import java.util.HashMap;
import java.util.Set;
import java.util.Vector;
import java.util.Iterator;
import java.io.Serializable;
import java.io.File;
import java.io.FileOutputStream;

/** I hate the name of this class, hate it, hate it, hate it.<p>
 * It's actually more like a model of an MVC pattern. Although, if I knew then,
 * what I know now, I'd merely pass around the ClassDef and TableDef objects
 * directly, this class is just adding a level of abstraction that doesn't buy
 * me anything in terms of functionality OR good OO design. sigh.<p>
 * Ok, so this class "holds" the list of classes and tables and is used by
 * various other objects within the dbMapper program to get info on these.
 * It also maintains a list of mappings created.<p>
 * BTW, Did I mention that I really hate the name of this class <p>
 *
 * @author Larry
 */
public class SourceProcessor implements Serializable{

    /** All classes for list enumeration
     */    
    public static final int CLASS_SELECTION_ALL = 1;
    /** All mapped classes for list enumeration
     */    
    public static final int CLASS_SELECTION_MAPPED = 2;
    /** All unmapped classes for list enumeration
     */    
    public static final int CLASS_SELECTION_UNMAPPED = 3;
    
    /** All tables for list enumeration
     */    
    public static final int TABLE_SELECTION_ALL = 1;
    /** All mapped tables for list enumeration
     */    
    public static final int TABLE_SELECTION_MAPPED = 2;
    /** All unmapped tables for list enumeration
     */    
    public static final int TABLE_SELECTION_UNMAPPED = 3;
    
    /** The list of classes to map
     */    
    TreeMap myClassMap=null;
    /** The list of tables to map
     */    
    TreeMap myTableMap=null;
    
    /** The list of mappings defined
     */    
    TreeMap mappedAssocs=null;
    
    /** The mode of class list selection
     */    
    private int class_selection_mode = CLASS_SELECTION_ALL;
    /** The mode of table list selection
     */    
    private int table_selection_mode = TABLE_SELECTION_ALL;

    /** t/f if classes and tables have been loaded (ie this is a valid processor)
     */    
    private boolean loaded=false;
    
    /** Creates a new instance of SourceProcessor */
    public SourceProcessor() {
        mappedAssocs=new TreeMap();
    }

    /** establish connections using the configuration object and fetch the class
     * list and table list.
     * @param conf the configuration object
     * @throws ConfigException error during configuration loading
     */
    public void load(Config conf) throws ConfigException {
        DbAccess dbaccess=new DbAccess(conf);
        dbaccess.load();
        myTableMap=dbaccess.getTableMap();
        ClassFinder sources=new ClassFinder(conf);
        sources.load();
        myClassMap=sources.getClassMap();
        
        mappedAssocs=new TreeMap();
        loaded=true;
    }
    
    /** t/f if this class has loaded classes & tables
     * @return t/f flag
     */    
    public boolean isLoaded(){ return loaded; }
    
    /** Set the class list display mode
     * @param mode The enumeration to select how class lists are returned
     */    
    public void setClassMode(int mode){
        class_selection_mode=mode;
    }
    /** Set the table list display mode
     * @param mode The enumeration to select how table lists are returned
     */    
    public void setDbTabMode(int mode){
        table_selection_mode=mode;
    }
    
    /** return list of classes
     * @return class list
     */
    public String[] getClassList(){
        return this.getClassList(class_selection_mode);
    }

    /** return list of classes with style of selection (all, unmapped, mapped)
     * specified.
     * @param mode selection mode
     * @return class list
     */
    public String[] getClassList(int mode){
        if ( myClassMap == null ) return null;
        TreeSet returnSet=new TreeSet();
        Iterator classlist=myClassMap.keySet().iterator();
        while ( classlist.hasNext() ){
            String className=(String)classlist.next();
            ClassDef classDef=(ClassDef)myClassMap.get(className);
            if ( mode == CLASS_SELECTION_UNMAPPED ){
                if ( classDef.isMapped() || classDef.isMod() ){
                    continue;
                }else{
                    returnSet.add((Object)className);
                }
            } else if ( mode == CLASS_SELECTION_MAPPED ){
                if ( classDef.isMapped() || classDef.isMod() ){
                    returnSet.add((Object)className);
                }else{
                    continue;
                }
            }else{
                returnSet.add((Object)className);
            }
        }
        return (String[])returnSet.toArray(Utils.emptystringarray);
    }
    
    /** Get list of attributes for the specified class
     * @param classname The classname to list attributes for
     * @return list of attributes
     */    
    public String[] getClassAttList(String classname){
        if ( myClassMap == null ) return null;
        if ( ! myClassMap.containsKey(classname) ){
               return null;
        }
        ClassDef classdef=(ClassDef)myClassMap.get(classname);
        return classdef.getAttrList();
    }

    /** Get the attribute for the specified class in a Pretty Print
     * format.
     * @param classname The classname of interest
     * @param attrname The attribute name of interest
     * @return The PP string
     */    
    public String getPPAttr(String classname, String attrname){
        if ( classname == null || attrname == null ) return null; 
        if ( ! myClassMap.containsKey(classname) ){
               return null;
        }
        ClassDef classdef=(ClassDef)myClassMap.get(classname);
        return classdef.getPPAttr(attrname);
    }
    
    
    /** Get the list of classes that extend the specified class
     * @param classname The classname
     * @return The list of extending classes
     */    
    public String[] getClassExtList(String classname){
        return ((ClassDef)myClassMap.get(classname)).getExtList();
    }
    
    
    /** return list of tables
     * @return table list
     */
    public String[] getTableList(){
        if ( myTableMap == null ) return null;
        Set keys=myTableMap.keySet();
        TreeSet returnSet=new TreeSet();
        Iterator tablelist=keys.iterator();
        while ( tablelist.hasNext() ){
            String tableName=(String)tablelist.next();
            TableDef tableDef=(TableDef)myTableMap.get(tableName);
            if ( table_selection_mode == TABLE_SELECTION_UNMAPPED ){
                if ( tableDef.isMapped() || tableDef.isMod() ){
                    continue;
                }else{
                    returnSet.add((Object)tableName);
                }
            } else if ( table_selection_mode == TABLE_SELECTION_MAPPED ){
                if ( tableDef.isMapped() || tableDef.isMod() ){
                    returnSet.add((Object)tableName);
                }else{
                    continue;
                }
            }else{
                returnSet.add((Object)tableName);
            }
        }
        return (String[])returnSet.toArray(Utils.emptystringarray);
    }
    
    /** get the list of columns for a specific table
     * @param tablename the tablename
     * @return list of columns
     */    
    public String[] getDbColList(String tablename){
        if ( myTableMap == null ) return null;
        if ( ! myTableMap.containsKey(tablename) ){
               return null;
        }
        TableDef table=(TableDef)myTableMap.get(tablename);
        return table.getColList();
    }
    
    /** Get the column for the specified table in a Pretty Print
     * format.
     * @param tablename the tablename of interest
     * @param colname the column name of interest
     * @return the PP string
     */    
    public String getPPCol(String tablename, String colname){
        //System.out.println("SP: getPPCol: " + tablename + " |" + colname);
        if ( tablename == null || colname == null ) return null; 
        if ( ! myTableMap.containsKey(tablename) ){
               return null;
        }
        TableDef table=(TableDef)myTableMap.get(tablename);
        return table.getPPCol(colname);
    }
    
    /** Set the specified class mode to be "in use"
     * @param classname the class name of interest
     */    
    public void setClassInUse(String classname){
        ClassDef classDef=(ClassDef)myClassMap.get(classname);
        if ( classDef == null ) return;
        classDef.setMod();
    }
    /** Unset the specified class mode from "in use" to "not in use"
     * @param classname the class name of interest
     */    
    public void unsetClassInUse(String classname){
        ClassDef classDef=(ClassDef)myClassMap.get(classname);
        if ( classDef == null ) return;
        classDef.setMod(false);
    }
    
    /** Set the specified table mode to "in use"
     * @param tablename the table name of interest
     */    
    public void setTableInUse(String tablename){
        TableDef tableDef=(TableDef)myTableMap.get(tablename);
        if ( tableDef == null ) return;
        tableDef.setMod();
    }
    /** Unset the specified table mode from "in use" to "not in use"
     * @param tablename the table name of interest
     */    
    public void unsetTableInUse(String tablename){
        TableDef tableDef=(TableDef)myTableMap.get(tablename);
        if ( tableDef == null ) return;
        tableDef.setMod(false);
    }

    /** is the specified tablename-column mapped?
     * @param tablename the tablename of interest
     * @param col the columnname of interest
     * @return t/f if mapped
     */    
    public boolean isTableColMapped(String tablename, String col ){
        if ( tablename == null || col == null ) return false; 
        if ( ! myTableMap.containsKey(tablename) ){
               return false;
        }
        TableDef tableDef=(TableDef)myTableMap.get(tablename);
        return tableDef.isColMapped(col);
    }
    
    /** is the specified classname-attribute mapped ?
     * @param classname the classname of interest
     * @param attr the attribute name of interest
     * @return t/f if mapped
     */    
    public boolean isClassAttMapped(String classname, String attr ){
        if ( classname == null || attr == null ) return false; 
        if ( ! myClassMap.containsKey(classname) ){
               return false;
        }
        ClassDef classdef=(ClassDef)myClassMap.get(classname);
        return classdef.isAttrMapped(attr);
    }

    
    /** Add a new mapping pair to the list of mappings.
     * @param map the new map to add to the mapping list
     */    
    public void addMapping(MappingDescriptor map){
        mappedAssocs.put(map.getClassname(), map);
        ClassDef classmapped = map.getClassDef();
        TableDef tablemapped = map.getTableDef();
        classmapped.setMapped();
        tablemapped.setMapped();
        myClassMap.put(classmapped.getName(), classmapped);
        myTableMap.put(tablemapped.getName(), tablemapped);
    }
    
    /** Get a cloned ClassDef object for the specified class.
     * (We want a clone so we can modify it, but then if we decide to
     * ignore the changes we make, all we need to do is dump the clone)
     * @param classnam the classname of interest
     * @return the cloned ClassDef
     */    
    public ClassDef getClassDefClone(String classnam){
        ClassDef classDef=(ClassDef)myClassMap.get(classnam);
        try {
            return (ClassDef)classDef.clone();
        }catch (Exception e){
            System.out.println("ClassDef Clone problem: " + e);
            e.printStackTrace();
            return null;
        }
    }
    
    /** Get a cloned TableDef object for the specified table.
     * (We want a clone so we can modify it, but then if we decide to
     * ignore the changes we make, all we need to do is dump the clone)
     * @param tablenam the table of interest
     * @return the cloned TableDef object
     */    
    public TableDef getTableDefClone(String tablenam){
        TableDef tableDef=(TableDef)myTableMap.get(tablenam);
        try {
            return (TableDef)tableDef.clone();
        }catch (Exception e){
            System.out.println("TableDef Clone problem: " + e);
            e.printStackTrace();
            return null;
        }
    }
    
    /** get the classDef object for the specified class
     * @param classnam the classname of interest
     * @return the classDef object
     */    
    public ClassDef getClassDef(String classnam){
        return (ClassDef)myClassMap.get(classnam);
    }
    
    /** get the TableDef object for the specified tablename
     * @param tablenam the tablename of interest
     * @return the tableDef
     */    
    public TableDef getTableDef(String tablenam){
        return (TableDef)myTableMap.get(tablenam);
    }

    /** get the xml mapping source for the current list of mappings
     * @return the xml mapping code
     */    
    public String getMappingSource(){
        StringBuffer results=new StringBuffer();
        Set entries=mappedAssocs.keySet();
        Iterator mapped=entries.iterator();
        while ( mapped.hasNext() ){
            ClassTableMap ctmap = (ClassTableMap)mappedAssocs.get((String)mapped.next());
            results.append(Utils.classMapXmlCode(ctmap));
        }
        return results.toString();
    }
    
    /** Get the mapping entry for the specified classname
     * @param classname the classname of interest
     * @return The mapping
     */    
    public ClassTableMap getMappedEntry(String classname){
        Set entries=mappedAssocs.keySet();
        Iterator mapped=entries.iterator();
        while ( mapped.hasNext() ){
            ClassTableMap ctmap = (ClassTableMap)mappedAssocs.get((String)mapped.next());
            if ( ctmap.getClassname().equals(classname) ) return ctmap;
        }
        return null;
    }
        
    
    /** Save the current mappings to a hierarchy of one file per class (ala javac)
     *
     * @param dir the top level directory to explode into
     */    
    public void saveMappingHier(File dir){
        Set keys=mappedAssocs.keySet();
        Iterator mapped=keys.iterator();
        while ( mapped.hasNext() ){
            ClassTableMap ctmap = (ClassTableMap)mappedAssocs.get((String)mapped.next());
            String classname = ctmap.getClassname();
            String root=classname.substring(0,classname.lastIndexOf('.'));
            String filenam=classname.substring(classname.lastIndexOf('.')+1) + ".xml";
            File outdir=new File(dir, root.replace('.','/'));
            File outfile=new File(outdir, filenam);
            try{
               outdir.mkdirs();
               outfile.createNewFile();
               String code=Utils.classMapXmlCode(ctmap);
               FileOutputStream out=new FileOutputStream(outfile);
               out.write(code.getBytes());
               out.close();
           } catch (Exception e){
                System.out.println("Write exception to " + outfile.getAbsolutePath() + 
                             " exc:" + e);
                continue;
           }
        }
        return;
    }     
}
