/*
 * TableDef.java
 *
 * Created on April 11, 2003, 8:17 AM
 */

package dbmapper;

import java.util.Vector;
import java.util.TreeMap;
import java.util.Set;
import java.io.Serializable;
/** This class maintains info about a single database table
 * including it's name, list of columns in this table, and a reference
 * to an entity which this class maps to.
 *
 * @author Larry
 * @see dbmapper.ClassDef
 */
public class TableDef implements Serializable, Cloneable{
    
    /** The name of this table
     */    
    private String name;
    
    /** The columns in this table
     */    
    private java.util.TreeMap cols;
    
    /** The object this table is mapped to
     */    
    private MappedReference obj = null;
    
    /** t/f flag if mapped
     */    
    private boolean mapped=false;
    /** t/f if a mod is in progress on this table
     */    
    private boolean modinprogress=false;
    
    public String getName(){return name; }
    
    public boolean isMapped(){ return mapped;}
    public void setMapped(){ mapped=true; }
    public void setMapped(boolean mapflag){ mapped=mapflag;}
    
    public boolean isMod(){ return modinprogress;}
    public void setMod(){ modinprogress=true; }
    public void setMod(boolean modflag){ modinprogress=modflag;}   
    
    /** Creates a new instance of TableDef
     * @param tabnam The name of the table
     */
    public TableDef(String tabnam) {
        name=tabnam;
        cols=new TreeMap();
    }
    
    public Object clone() throws CloneNotSupportedException{
        return super.clone();
    }
    
    public void addCol(String colnam, String coltyp) {
        if ( colnam == null || coltyp == null ) return;
        cols.put(colnam, new ColDef(colnam,coltyp));
        return;
    }
    
    public int numCols(){
        return cols.size();
    }

    public String[] getColList(){
        Set keyset=cols.keySet();
        return (String[])keyset.toArray(Utils.emptystringarray);
    }
    
    public boolean isColMapped(String colname){
        if ( colname == null ) return false;
        if ( cols.containsKey(colname) ){
            return ((ColDef)cols.get(colname)).isMapped();
        }
        return false;
    }
    
    public String getPPCol(String colname){
        if ( colname == null ) return null;
        if ( cols.containsKey(colname) ){
            return ((ColDef)cols.get(colname)).getPPstr();
        }
        return null;
    }
    
    public ColDef getCol(String colname){
        if ( colname == null ) return null;
        if ( cols.containsKey(colname) ){
            return (ColDef)cols.get(colname);
        }
        return null;
    }
    
    public void setMappedTo(MappedReference o){
        obj=o;
    }
    
    public MappedReference getMappedTo(){
        return obj;
    }
}
