/*
 * Utils.java
 *
 * Created on April 10, 2003, 6:30 PM
 */
package dbmapper;

import java.util.Properties;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Set;
import java.util.Iterator;
import java.io.File;
/** Some misc utils
 * @author Larry
 */
public class Utils {
    
    /** last directory active by the FileChooser
     */    
    public static File lastDir = new File(System.getProperty("user.dir"));
    
    /** empty String array for toArray calls
     */    
    public static final String[] emptystringarray={};
    
    /** Creates a new instance of Utils */
    public Utils() {
    }
    
     /** Reads in properties from a property file.
      *
      * @param propertyFile	name of the property file
      * @return xml mapping code
      */
    public static boolean loadProperties (String propertyFile) {
        Properties props = new Properties();
        try {
            InputStream is = ClassLoader.getSystemResourceAsStream(propertyFile);
            if ( is == null ) throw new Exception("Couldn't get inputstream of " + propertyFile);
            props.load (is);
            is.close();
        } catch (Exception exc) {
            System.out.println ("Failed to load " + propertyFile + " " + exc.toString());
            return false;
        }

        // Set system properties based on each property just loaded.
        for (Enumeration e = props.propertyNames(); e.hasMoreElements(); ){
            String propName = (String) e.nextElement();
            String propValue = props.getProperty (propName);
            System.setProperty (propName, propValue);
        }
        return true;
    }
  
    /** create mapping xml code for class
     * @param myMap class mapping info
     * @return xml mapping code
     */    
    public static String classMapXmlCode(ClassTableMap myMap){
       ClassDef myClassDef=myMap.getClassDef();
       TableDef myTableDef=myMap.getTableDef();
       // create the xml mapping code for this class/table mapping
       StringBuffer codeOut=new StringBuffer();
       String fullName=myClassDef.getName();
       fullName.replace('/','.');
       codeOut.append("<!-- ------------------------------------------------------------------- ->\n");
       codeOut.append("<!-- ------------------------------------------------------------------- ->\n");
       codeOut.append("<!-- ").append(fullName).append(" -->\n");
       codeOut.append("<!-- ------------------------------------------------------------------- ->\n");
       codeOut.append("<class-descriptor class=\"").append(fullName).append("\"\n");
       codeOut.append("                  table=\"").append(myTableDef.getName()).append("\">\n");
       codeOut.append("\n");
   
       // do the extents stuff here
       // LRS WORK
       /**      <extent-class class-ref="org.apache.ojb.broker.BookArticle" />
                <extent-class class-ref="org.apache.ojb.broker.CdArticle" />
        */
       String[] exts=myClassDef.getExtList();
       for ( int i=0; i<exts.length; i++){
                  codeOut.append("<extent-class class-ref=\"");
                  codeOut.append(exts[i]).append("\"/>\n");
       }
       
       int i=1;
       for (Enumeration e = myMap.getMapping() ; e.hasMoreElements() ;) {
           MappedReference item=(MappedReference)e.nextElement();
           // ignore dud links
           switch (item.getMapType() ){
               case  ClassTableMap.MAP_TYPE_NA:
               case  ClassTableMap.MAP_TYPE_DATTR: 
               case  ClassTableMap.MAP_TYPE_DCOL:      continue; // ignore;
                      
               case  ClassTableMap.MAP_TYPE_PK:  {
               codeOut.append("<field-descriptor \n");
               codeOut.append("     id=\"").append(i).append("\"\n");
               codeOut.append("     name=\"").append(item.getName()).append("\"\n");
               codeOut.append("     column=\"");
                codeOut.append(((MappedReference)item.getMappedTo()).getName());
                codeOut.append("\"\n");
               codeOut.append("     jdbc-type=\"BIGINT\"\n");
               MappingProperties props=item.getMapProp();
               Set propSet=props.keySet();
               Iterator propsIt=propSet.iterator();
               while ( propsIt.hasNext() ){
                   String key=(String)propsIt.next();
                   if ( "".equals((String)props.get(key)) ) continue;
                   codeOut.append("     ").append(key).append("=\"");
                   codeOut.append(props.get(key)).append("\"\n");
               }
               codeOut.append("     >\n");
               codeOut.append("</field-descriptor>\n");
               i++;
               codeOut.append("\n");
               continue;
               }
               
               case ClassTableMap.MAP_TYPE_PRIM: {
               codeOut.append("<field-descriptor \n");
               codeOut.append("     id=\"").append(i).append("\"\n");
               codeOut.append("     name=\"").append(item.getName()).append("\"\n");
               codeOut.append("     column=\"");
                codeOut.append(((MappedReference)item.getMappedTo()).getName());
                codeOut.append("\"\n");
               codeOut.append("     jdbc-type=\"");
                codeOut.append(Utils.typeConversion(item.getType())).append("\"\n");
               MappingProperties props=item.getMapProp();
               Set propSet=props.keySet();
               Iterator propsIt=propSet.iterator();
               while ( propsIt.hasNext() ){
                   String key=(String)propsIt.next();
                   if ( "".equals((String)props.get(key)) ) continue;
                   codeOut.append("     ").append(key).append("=\"");
                   codeOut.append(props.get(key)).append("\"\n");
               }
               codeOut.append("     >\n");
               codeOut.append("</field-descriptor>\n");
               i++;
               codeOut.append("\n");
               continue;
               }
               
               case ClassTableMap.MAP_TYPE_OBJREF: {
               codeOut.append("<field-descriptor \n");
               codeOut.append("     id=\"").append(i).append("\"\n");
               codeOut.append("     name=\"").append(item.getName()).append("_Id\"\n");
               codeOut.append("     column=\"");
                codeOut.append(((MappedReference)item.getMappedTo()).getName());
                codeOut.append("\"\n");
               codeOut.append("     jdbc-type=\"BIGINT\"\n");
               MappingProperties props=item.getMapProp();
               Set propSet=props.keySet();
               Iterator propsIt=propSet.iterator();
               while ( propsIt.hasNext() ){
                   String key=(String)propsIt.next();
                   if ( "".equals((String)props.get(key)) ) continue;
                   if ( key.startsWith("RD") ) continue;
                   codeOut.append("     ").append(key).append("=\"");
                   codeOut.append(props.get(key)).append("\"\n");
               }
               codeOut.append("     >\n");
               codeOut.append("</field-descriptor>\n");
               i++;
               codeOut.append("\n");
               codeOut.append("<reference-descriptor \n");
               codeOut.append("     id=\"").append(i).append("\"\n");
               codeOut.append("     name=\"").append(item.getName()).append("\"\n");
               codeOut.append("     class-ref=\"").append(cleanClassName(item.getType()));
               codeOut.append("\"\n");
               propsIt=propSet.iterator();
               while ( propsIt.hasNext() ){
                   String key=(String)propsIt.next();
                   if ( "".equals((String)props.get(key)) ) continue;
                   if ( ! key.startsWith("RD") ) continue;
                   codeOut.append("     ").append(key.substring(2)).append("=\"");
                   codeOut.append(props.get(key)).append("\"\n");
               }
                codeOut.append("     >\n");
               codeOut.append("   <foreignkey field-ref=\"");
                codeOut.append(item.getName()).append("_Id\">\n");
               codeOut.append("</reference-descriptor>\n");
               i++;
               codeOut.append("\n");
               continue;
               }

               case ClassTableMap.MAP_TYPE_COLLECT:{
               codeOut.append("<collection-descriptor \n");
               codeOut.append("     id=\"").append(i).append("\"\n");
               codeOut.append("     name=\"").append(item.getName()).append("\"\n");
               codeOut.append("     element-class-ref=\"");
                codeOut.append(cleanClassName(item.getType())).append("\"\n");
               codeOut.append("     <inverse-foreignkey field-ref=\"");
                codeOut.append(item.getMappedTo().getName()).append("\"/>\n");
               MappingProperties props=item.getMapProp();
               Set propSet=props.keySet();
               Iterator propsIt=propSet.iterator();
               while ( propsIt.hasNext() ){
                   String key=(String)propsIt.next();
                   if ( "".equals((String)props.get(key)) ) continue;
                   codeOut.append("     ").append(key).append("=\"");
                   codeOut.append(props.get(key)).append("\"\n");
               }
               codeOut.append("     >\n");
               codeOut.append("</collection-descriptor>\n");
               i++;
               codeOut.append("\n");
               continue;
               }
               
               case ClassTableMap.MAP_TYPE_MANY:{
               MappedReference mapto=(MappedReference)item.getMappedTo();
               if ( ! (mapto instanceof ManyMap) ){
                   System.out.println("Bad many-map reference" + item + " " + mapto);
                   continue;
               }
               ManyMap manymap=(ManyMap)mapto;
               codeOut.append("<collection-descriptor \n");
               codeOut.append("     id=\"").append(i).append("\"\n");
               codeOut.append("     name=\"").append(item.getName()).append("\"\n");
               codeOut.append("     element-class-ref=\"");
                codeOut.append(cleanClassName(item.getType())).append("\"\n");
               MappingProperties props=item.getMapProp();
               Set propSet=props.keySet();
               Iterator propsIt=propSet.iterator();
               while ( propsIt.hasNext() ){
                   String key=(String)propsIt.next();
                   if ( "".equals((String)props.get(key)) ) continue;
                   codeOut.append("     ").append(key).append("=\"");
                   codeOut.append(props.get(key)).append("\"\n");
               }
               codeOut.append("     indirection-table=\"");
               codeOut.append(manymap.getTablename()).append("\">\n");
               codeOut.append("     <fk-pointing-to-this-class column=\"");
                codeOut.append(manymap.getFktome()).append("\"/>\n");
               codeOut.append("     <fk-pointing-to-element-class column=\"");
                codeOut.append(manymap.getFktooth()).append("\"/>\n");
               codeOut.append("</collection-descriptor>\n");
               i++;
               codeOut.append("\n");
               continue;
               }
           }
       }
       codeOut.append("</class-descriptor>\n\n");
       
       return codeOut.toString();
       
   }

   // better to load this into a static hashtable... WORK LRS
   // return jdbc type given a java type
    /** convert java type to jdbc type
     * @param javaType java type specifier
     * @return jdbc type specifier
     */    
    public static String typeConversion(String javaType){
        if ( javaType == null || javaType.equals("") ) return null;
        if ( javaType.equals("java.lang.String") ) return "VARCHAR";
        if ( javaType.equals("long") ) return "BIGINT";
        if ( javaType.equals("Float") ) return "REAL";
        if ( javaType.equals("java.lang.Float") ) return "REAL";
        if ( javaType.equals("java.lang.Long") ) return "BIGINT";
        if ( javaType.equals("java.lang.Integer") ) return "INTEGER";
        if ( javaType.equals("java.lang.boolean") ) return "BIT";
        if ( javaType.equals("java.lang.double") ) return "FLOAT";
        return "<<not-mapped>>";
    }
        
        /* Types mapping to complete - 
         * JDBC Type   Java Type     
         CHAR   String     
         VARCHAR   String     
         LONGVARCHAR   String     
         NUMERIC   java.math.BigDecimal     
         DECIMAL   java.math.BigDecimal     
         BIT   boolean     
         TINYINT   byte     
         SMALLINT   short     
         INTEGER   int     
         BIGINT   long     
         REAL   float     
         FLOAT   double     
         DOUBLE   double     
         BINARY   byte[]     
         VARBINARY   byte[]     
         LONGVARBINARY   byte[]     
         DATE   java.sql.Date     
         TIME   java.sql.Time     
         TIMESTAMP   java.sql.Timestamp     
         CLOB   Clob     
         BLOB   Blob     
         ARRAY   Array     
         DISTINCT   mapping of underlying type     
         STRUCT   Struct     
         REF   Ref     
         JAVA_OBJECT   underlying Java class
         */
    
    /** Clean a classname as returned from a call to Class.getName(). The only problem
     * I needed to fix was with arrays, Class returns [Lclassname; as an overload
     * for arrays, so this method strips the L and ; and returns the rest.<p>
     * Oh, I also needed to strip a trailing "[]" which I added in the ClassFinder
     * when Class.isArray() returned true.<p>
     * @param ugly The ugly class name
     * @return The pretty classname
     */    
    private static String cleanClassName(String ugly){
        if ( ugly == null ) return null; //take that
        if ( ugly.startsWith("[L") ) ugly=ugly.substring(2);
        if ( ugly.endsWith(";[]" ) ) ugly=ugly.substring(0,ugly.length()-3);
        return ugly;
    }
}
